/************************************************************************/
/* shttp.h - Simple HTTP protocol connection header			*/
/************************************************************************/
/*
 *  (c)2002 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/

#ifndef _SHTTP_H_
#define _SHTTP_H_

#include "hash.h"

#define HTTP_MESSAGE_FLAG_REQUEST		0x00000001
#define HTTP_MESSAGE_FLAG_RESPONSE		0x00000002

typedef enum {
	HTTPTYPENONE = 0,
	HTTPTYPEGET,		/* Request */
	HTTPTYPEPUT,		/* Request */
	HTTPTYPEPOST,		/* Request */
	HTTPTYPEHEAD,		/* Request */
	HTTPTYPERESPONSE,	/* Response */
	HTTPTYPEMAX
	} HttpMessageType;


typedef int (*HttpDataFunction)(void *connectdata,
		char *buffer,
		int *bytes);

typedef struct _HttpConnection HttpConnection;
typedef struct _HttpMessage HttpMessage;

struct _HttpConnection {
	void *connectdata;
	HttpDataFunction write_function;
	HttpDataFunction read_function;
	int flags;
	int rc;
	};

struct _HttpMessage {
	HttpMessageType messagetype;
	char *url;
	SHash *headerhash;
	char *entity;
	int entitylen;
	char *httpversion;
	int httprc;
	char *httprcmsg;
	int flags;
	};


/************************************************************************/
/*
 * Function: shttp_connection_new
 * Purpose:  Create a new connection structure, to send/receive HTTP.
 * Argument: connectdata - Pointer to the connection data used.
 * Returns:  A pointer to an HttpConnection structure on success.
 * Returns:  NULL - If an error occured.
 */
/************************************************************************/
int shttp_connection_new(HttpConnection **hcp, void *connectdata);


/************************************************************************/
/*
 * Function: shttp_connection_set_write_callback
 * Purpose:  Associate a writer function with the given HttpConnection.
 * Argument: hc - Pointer to a HttpConnection structure.
 * Argument: write_function - the writer function for this connection.
 * Returns:  0 - On success.
 * Returns:  -1 - If an error occured.
 */
/************************************************************************/
int shttp_connection_set_write_callback(HttpConnection *hc,
		HttpDataFunction write_function);


/************************************************************************/
/*
 * Function: shttp_connection_set_read_callback
 * Purpose:  Associate a reader function with the given HttpConnection.
 * Argument: hc - Pointer to a HttpConnection structure.
 * Argument: write_function - the reader function for this connection.
 * Returns:  0 - On success.
 * Returns:  -1 - If an error occured.
 */
/************************************************************************/
int shttp_connection_set_read_callback(HttpConnection *hc,
		HttpDataFunction read_function);


/************************************************************************/
/*
 * Function: shttp_connection_digest_message
 * Purpose:  Digest HTTP data from the given connection.
 * Argument: hc - A pointer to an HttpConnection structure to be used.
 * Returns:  Pointer to an HttpMessage structure on success.
 * Returns:  NULL if an error occured.
 */
/************************************************************************/
int shttp_connection_digest_message(HttpConnection *hc, HttpMessage **hmp);


/************************************************************************/
/*
 * Function: shttp_connection_send_message
 * Purpose:  Encapsulate & transmit HTTP data to the given connection.
 * Argument: hc - A pointer to an HttpConnection structure to be used.
 * Argument: hm - Pointer to an HttpMessage structure to transmit.
 * Returns:  0 - On success.
 * Returns:  -1 - If an error occured.
 */
/************************************************************************/
int shttp_connection_send_message(HttpConnection *hc, HttpMessage *hm);

/************************************************************************/
/*
 * Function: shttp_connection_free
 * Purpose:  Frees an HttpConnection structure and it's contents.
 * Argument: hc - Pointer to an HttpConnection structure.
 * Returns:  0 - On success
 * Returns: -1 - If an error occured.
 */
/************************************************************************/
int shttp_connection_free(HttpConnection **hc);


/************************************************************************/
/*
 * Function: shttp_message_new
 * Purpose:  Allocate a new HttpMessage structure.
 * Argument: None.
 * Returns:  On success, a pointer to the newly allocated HttpMessage.
 * Returns:  Null if an error occured.
 */
/************************************************************************/
int shttp_message_new(HttpMessage **hmp);


/************************************************************************/
/*
 * Function: shttp_message_free
 * Purpose:  Frees an HttpMessage structure and it's contents.
 * Argument: hm - Pointer to an HttpMessage structure pointer.
 * Returns:  0 - On success
 * Returns:  -1 - If an error occured.
 */
/************************************************************************/
int shttp_message_free(HttpMessage **hmp);


/************************************************************************/
/*
 * Function: shttp_message_set_http_version
 * Purpose:  Set HTTP version string for the message.
 * Argument: hm - Pointer to an HttpMessage structure.
 * Argument: hv - Pointer to an HTTP version string.
 * Returns:  0 - On success
 * Returns:  -1 - If an error occured.
 */
/************************************************************************/
int shttp_message_set_http_version(HttpMessage *hm, char *hv);


/************************************************************************/
/*
 * Function: shttp_message_set_message_type
 * Purpose:  Set the HTTP request type(GET,POST, PUT, etc)
 * Argument: hm - Pointer to an HttpMessage structure.
 * Argument: hrt - The request type.
 * Returns:  0 - On success
 * Returns:  -1 - If an error occured.
 */
/************************************************************************/
int shttp_message_set_message_type(HttpMessage *hm, HttpMessageType hrt);


/************************************************************************/
/*
 * Function: shttp_message_set_request_url
 * Purpose:  Set the HTTP request url.
 * Argument: hm - Pointer to an HttpMessage structure.
 * Argument: hurl - The request url.
 * Returns:  0 - On success
 * Returns:  -1 - If an error occured.
 */
/************************************************************************/
int shttp_message_set_request_url(HttpMessage *hm, char *hurl);


/************************************************************************/
/*
 * Function: shttp_message_set_response_code
 * Purpose:  Set the HTTP response return code.
 * Argument: hm - Pointer to an HttpMessage structure.
 * Argument: hrc - The response code
 * Returns:  0 - On success
 * Returns:  -1 - If an error occured.
 */
/************************************************************************/
int shttp_message_set_response_code(HttpMessage *hm, int hrc);


/************************************************************************/
/*
 * Function: shttp_message_set_response_message
 * Purpose:  Set the HTTP response return code message.
 * Argument: hm - Pointer to an HttpMessage structure.
 * Argument: hrc - The response code message
 * Returns:  0 - On success
 * Returns:  -1 - If an error occured.
 */
/************************************************************************/
int shttp_message_set_response_message(HttpMessage *hm, char *hrcm);


/************************************************************************/
/*
 * Function: shttp_message_add_header
 * Purpose:  Add a header to the message.
 * Argument: hm - Pointer to an HttpMessage structure.
 * Argument: header - The header name to add.
 * Argument: value - The value associated with the header.
 * Returns:  0 - On success
 * Returns:  -1 - If an error occured.
 */
/************************************************************************/
int shttp_message_add_header(HttpMessage *hm, char *header, char *value);


/************************************************************************/
/*
 * Function: shttp_message_add_entity
 * Purpose:  Add an entity to the message.(Adds a Content-Length header)
 * Argument: hm - Pointer to an HttpMessage structure.
 * Argument: entity - Pointer to the entity buffer.
 * Argument: entitylen - The length of the entity, in bytes.
 * Argument: mimetype - Mimetype string for the entity. If NULL, defaults
 *           to 'text/plain'. Adds Content-Type header.
 * Returns:  0 - On success
 * Returns:  -1 - If an error occured.
 */
/************************************************************************/
int shttp_message_add_entity(HttpMessage *hm,
		char *entity,
		int entitylen,
		char *mimetype);


/************************************************************************/
/*
 * Function: shttp_message_get_entity
 * Purpose:  Return a pointer to the entity data.
 * Argument: hm - Pointer to an HttpMessage structure.
 * Argument: entity - void pointer-pointer to be set.
 * Argument: entitylen - int pointer to be set.
 * Returns:  On success, 0 is returned.
 * Returns:  On failure, >0 is returned.
 */
/************************************************************************/
int shttp_message_get_entity(HttpMessage *hm, void **entity,
			int *entitylen);


/************************************************************************/
/*
 * Function: shttp_message_get_entity_size
 * Purpose:  Returns the length of the entity data.
 * Argument: hm - Pointer to an HttpMessage structure.
 * Returns:  On success, returns the length of the entity data in bytes.
 * Returns:  On failure, returns 0.
 */
/************************************************************************/
int shttp_message_get_entity_size(HttpMessage *hm);



/************************************************************************/
/*
 * Function: shttp_message_parse_headerline
 * Purpose:  Parse a headerline and add the header to the HttpMessage
 * Argument: hm - Pointer to an HttpMessage structure.
 * Argument: headerline - Pointer to the headerline to be parse
 * Returns:  0 - On success
 * Returns:  >=1 - If an error occured.
 */
/************************************************************************/
int shttp_message_parse_headerline(HttpMessage *hm,
			char *headerline);


/************************************************************************/
/*
 * Function: shttp_message_get_header_value
 * Purpose:  Return a pointer to the value of the specified header.
 * Argument: hm - Pointer to an HttpMessage structure.
 * Argument: header - The header name to return the value for.
 * Returns:  On success, a pointer to the header value.
 * Returns:  On failure, returns NULL.
 */
/************************************************************************/
void *shttp_message_get_header_value(HttpMessage *hm,
			char *header);


/************************************************************************/
/*
 * Function: shttp_message_get_url
 * Purpose:  Return a pointer to the URL string.
 * Argument: hm - Pointer to an HttpMessage structure.
 * Argument: urlp - String pointer-pointer to receive the URL string.
 * Returns:  On success, returns 0 and urlp points to the (const) URL string.
 * Returns:  On failure, returns >0
 */
/************************************************************************/
int shttp_message_get_url(HttpMessage *hm,
			char **urlp);



#endif
